@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """Aliases for the xonsh shell."""
 import os
 import shlex
@@ -33,41 +34,59 @@ def _ensure_source_foreign_parser():
     desc = "Sources a file written in a foreign shell language."
     parser = ArgumentParser('source-foreign', description=desc)
     parser.add_argument('shell', help='Name or path to the foreign shell')
-    parser.add_argument('filenames', nargs='+', help='file paths to source')
+    parser.add_argument('files_or_code', nargs='+',
+                        help='file paths to source or code in the target '
+                             'language.')
     parser.add_argument('-i', '--interactive', type=to_bool, default=True,
                         help='whether the sourced shell should be interactive',
                         dest='interactive')
     parser.add_argument('-l', '--login', type=to_bool, default=False,
                         help='whether the sourced shell should be login',
                         dest='login')
-    parser.add_argument('--envcmd', default='env', dest='envcmd', 
+    parser.add_argument('--envcmd', default='env', dest='envcmd',
                         help='command to print environment')
-    parser.add_argument('--aliascmd', default='alias', dest='aliascmd', 
+    parser.add_argument('--aliascmd', default='alias', dest='aliascmd',
                         help='command to print aliases')
     parser.add_argument('--extra-args', default=(), dest='extra_args',
-                        type=(lambda s: tuple(s.split())), 
+                        type=(lambda s: tuple(s.split())),
                         help='extra arguments needed to run the shell')
-    parser.add_argument('-s', '--safe', type=to_bool, default=True, 
+    parser.add_argument('-s', '--safe', type=to_bool, default=True,
                         help='whether the source shell should be run safely, '
                              'and not raise any errors, even if they occur.',
                         dest='safe')
-    parser.add_argument('--sourcer', default='source', dest='sourcer',
+    parser.add_argument('-p', '--prevcmd', default=None, dest='prevcmd',
+                        help='command(s) to run before any other commands, '
+                             'replaces traditional source.')
+    parser.add_argument('--postcmd', default='', dest='postcmd',
+                        help='command(s) to run after all other commands')
+    parser.add_argument('--funcscmd', default=None, dest='funcscmd',
+                        help='code to find locations of all native functions '
+                             'in the shell language.')
+    parser.add_argument('--sourcer', default=None, dest='sourcer',
                         help='the source command in the target shell language, '
                              'default: source.')
     _SOURCE_FOREIGN_PARSER = parser
     return parser
-    
+
 
 def source_foreign(args, stdin=None):
     """Sources a file written in a foreign shell language."""
     parser = _ensure_source_foreign_parser()
     ns = parser.parse_args(args)
-    prevcmd = '{0} {1}'.format(ns.sourcer, ' '.join(ns.filenames))
+    if ns.prevcmd is not None: 
+        pass  # don't change prevcmd if given explicitly
+    elif os.path.isfile(ns.files_or_code[0]):
+        # we have filename to source
+        ns.prevcmd = '{0} {1}'.format(ns.sourcer, ' '.join(ns.files_or_code))
+    elif ns.prevcmd is None:
+        ns.prevcmd = ' '.join(ns.files_or_code)  # code to run, no files
     foreign_shell_data.cache_clear()  # make sure that we don't get prev src
     fsenv, fsaliases = foreign_shell_data(shell=ns.shell, login=ns.login,
-                            interactive=ns.interactive, envcmd=ns.envcmd, 
-                            aliascmd=ns.aliascmd, extra_args=ns.extra_args,  
-                            safe=ns.safe, prevcmd=prevcmd)
+                            interactive=ns.interactive, envcmd=ns.envcmd,
+                            aliascmd=ns.aliascmd, extra_args=ns.extra_args,
+                            safe=ns.safe, prevcmd=ns.prevcmd,
+                            postcmd=ns.postcmd, funcscmd=ns.funcscmd,
+                            sourcer=ns.sourcer)
     # apply results
     env = builtins.__xonsh_env__
     denv = env.detype()
