@@ -1,5 +1,8 @@
-"""The xonsh built-ins. Note that this module is named 'built_ins' so as
-not to be confused with the special Python builtins module.
+# -*- coding: utf-8 -*-
+"""The xonsh built-ins.
+
+Note that this module is named 'built_ins' so as not to be confused with the
+special Python builtins module.
 """
 import os
 import re
@@ -10,16 +13,15 @@ import atexit
 import signal
 import inspect
 import builtins
-import subprocess
-from io import TextIOWrapper, StringIO
 from glob import glob, iglob
 from subprocess import Popen, PIPE, STDOUT
 from contextlib import contextmanager
-from collections import Sequence, MutableMapping, Iterable, namedtuple, \
-    MutableSequence, MutableSet
+from collections import Sequence, MutableMapping, Iterable
 
-from xonsh.tools import suggest_commands, XonshError, ON_POSIX, ON_WINDOWS, \
-    string_types
+from xonsh.tools import (
+    suggest_commands, XonshError, ON_POSIX, ON_WINDOWS, string_types,
+    expandvars,
+)
 from xonsh.inspectors import Inspector
 from xonsh.environ import Env, default_env, locate_binary
 from xonsh.aliases import DEFAULT_ALIASES
@@ -31,14 +33,14 @@ from xonsh.foreign_shells import load_foreign_aliases
 ENV = None
 BUILTINS_LOADED = False
 INSPECTOR = Inspector()
-AT_EXIT_SIGNALS = (signal.SIGABRT, signal.SIGFPE, signal.SIGILL, signal.SIGSEGV, 
+AT_EXIT_SIGNALS = (signal.SIGABRT, signal.SIGFPE, signal.SIGILL, signal.SIGSEGV,
                    signal.SIGTERM)
 if ON_POSIX:
     AT_EXIT_SIGNALS += (signal.SIGTSTP, signal.SIGQUIT, signal.SIGHUP)
 
 
 def resetting_signal_handle(sig, f):
-    """Sets a new signal handle that will automaticallly restore the old value 
+    """Sets a new signal handle that will automatically restore the old value
     once the new handle is finished.
     """
     oldh = signal.getsignal(sig)
@@ -89,8 +91,10 @@ class Aliases(MutableMapping):
         # only once.
         if callable(value):
             if acc_args:  # Partial application
-                return lambda args, stdin=None: value(acc_args + args,
-                                                      stdin=stdin)
+                def _alias(args, stdin=None):
+                    args = [expand_path(i) for i in (acc_args + args)]
+                    return value(args, stdin=stdin)
+                return _alias
             else:
                 return value
         else:
@@ -105,6 +109,17 @@ class Aliases(MutableMapping):
                                        seen_tokens | {token},
                                        rest + acc_args)
 
+    def expand_alias(self, line):
+        """Expands any aliases present in line if alias does not point to a
+        builtin function and if alias is only a single command.
+        """
+        word = line.split(' ', 1)[0]
+        if word in builtins.aliases and isinstance(self.get(word), Sequence):
+            word_idx = line.find(word)
+            expansion = ' '.join(self.get(word))
+            line = line[:word_idx] + expansion + line[word_idx+len(word):]
+        return line
+
     #
     # Mutable mapping interface
     #
@@ -164,9 +179,9 @@ def superhelper(x, name=''):
 def expand_path(s):
     """Takes a string path and expands ~ to home and environment vars."""
     global ENV
-    if ENV is not None:
-        ENV.replace_env()
-    return os.path.expanduser(os.path.expandvars(s))
+    if ENV.get('EXPAND_ENV_VARS'):
+        s = expandvars(s)
+    return os.path.expanduser(s)
 
 
 def expand_case_matching(s):
@@ -319,9 +334,9 @@ def get_script_subproc_command(fname, args):
         raise PermissionError
 
     if ON_POSIX and not os.access(fname, os.R_OK):
-        # on some systems, some importnat programs (e.g. sudo) will have execute 
-        # permissions but not read/write permisions. This enables things with the SUID 
-        # set to be run. Needs to come before _is_binary() is called, because that 
+        # on some systems, some importnat programs (e.g. sudo) will have execute
+        # permissions but not read/write permisions. This enables things with the SUID
+        # set to be run. Needs to come before _is_binary() is called, because that
         # function tries to read the file.
         return [fname] + args
     elif _is_binary(fname):
@@ -332,7 +347,7 @@ def get_script_subproc_command(fname, args):
         # Windows can execute various filetypes directly
         # as given in PATHEXT
         _, ext = os.path.splitext(fname)
-        if ext.upper() in builtins.__xonsh_env__.get('PATHEXT', []):
+        if ext.upper() in builtins.__xonsh_env__.get('PATHEXT'):
             return [fname] + args
 
     # find interpreter
@@ -479,15 +494,12 @@ def run_subproc(cmds, captured=True):
         cmds = cmds[:-1]
     write_target = None
     last_cmd = len(cmds) - 1
-    prev = None
     procs = []
     prev_proc = None
     for ix, cmd in enumerate(cmds):
         stdin = None
-        stdout = None
         stderr = None
         if isinstance(cmd, string_types):
-            prev = cmd
             continue
         streams = {}
         while True:
@@ -525,7 +537,7 @@ def run_subproc(cmds, captured=True):
         alias = builtins.aliases.get(cmd[0], None)
         if (alias is None
             and builtins.__xonsh_env__.get('AUTO_CD')
-            and len(cmds)==1
+            and len(cmd) == 1
             and os.path.isdir(cmd[0])
             and locate_binary(cmd[0], cwd=None) is None):
             cmd.insert(0, 'cd')
@@ -585,7 +597,6 @@ def run_subproc(cmds, captured=True):
                     e += '\n' + suggest_commands(cmd, ENV, builtins.aliases)
                 raise XonshError(e)
         procs.append(proc)
-        prev = None
         prev_proc = proc
     for proc in procs[:-1]:
         try:
@@ -599,6 +610,12 @@ def run_subproc(cmds, captured=True):
             'obj': prev_proc,
             'bg': background
         })
+    if ENV.get('XONSH_INTERACTIVE') and not ENV.get('XONSH_STORE_STDOUT'):
+        # set title here to get current command running
+        try:
+            builtins.__xonsh_shell__.settitle()
+        except AttributeError:
+            pass
     if background:
         return
     if prev_is_proxy:
@@ -612,7 +629,7 @@ def run_subproc(cmds, captured=True):
         if prev_proc.stdout not in (None, sys.stdout):
             output = prev_proc.stdout.read()
         if captured:
-            # to get proper encoding from Popen, we have to 
+            # to get proper encoding from Popen, we have to
             # use a byte stream and then implement universal_newlines here
             output = output.decode(encoding=ENV.get('XONSH_ENCODING'),
                                    errors=ENV.get('XONSH_ENCODING_ERRORS'))
@@ -681,7 +698,7 @@ def load_builtins(execer=None):
     builtins.aliases.update(load_foreign_aliases(issue_warning=False))
     # history needs to be started after env and aliases
     # would be nice to actually include non-detyped versions.
-    builtins.__xonsh_history__ = History(env=ENV.detype(), #aliases=builtins.aliases, 
+    builtins.__xonsh_history__ = History(env=ENV.detype(), #aliases=builtins.aliases,
                                          ts=[time.time(), None], locked=True)
     lastflush = lambda s=None, f=None: builtins.__xonsh_history__.flush(at_exit=True)
     atexit.register(lastflush)
