@@ -1,14 +1,16 @@
+# -*- coding: utf-8 -*-
 """A (tab-)completer for xonsh."""
 import os
 import re
-import builtins
-import pickle
-import shlex
-import subprocess
+import ast
 import sys
+import shlex
+import pickle
+import builtins
+import subprocess
 
-from xonsh.built_ins import iglobpath
-from xonsh.tools import subexpr_from_unbalanced
+from xonsh.built_ins import iglobpath, expand_path
+from xonsh.tools import subexpr_from_unbalanced, get_sep, check_for_partial_string, RE_STRING_START
 from xonsh.tools import ON_WINDOWS
 
 
@@ -16,6 +18,40 @@ RE_DASHF = re.compile(r'-F\s+(\w+)')
 RE_ATTR = re.compile(r'(\S+(\..+)*)\.(\w*)$')
 RE_WIN_DRIVE = re.compile(r'^([a-zA-Z]):\\')
 
+
+def _path_from_partial_string(inp, pos=None):
+    if pos is None:
+        pos = len(inp)
+    partial = inp[:pos]
+    startix, endix, quote = check_for_partial_string(partial)
+    _post = ""
+    if startix is None:
+        return None
+    elif endix is None:
+        string = partial[startix:]
+    else:
+        if endix != pos:
+            _test = partial[endix:pos]
+            if not any(i == ' ' for i in _test):
+                _post = _test
+            else:
+                return None
+        string = partial[startix:endix]
+    end = re.sub(RE_STRING_START,'',quote)
+    _string = string
+    if not _string.endswith(end):
+        _string = _string + end
+    try:
+        val = ast.literal_eval(_string)
+    except SyntaxError:
+        return None
+    if isinstance(val, bytes):
+        env = builtins.__xonsh_env__
+        val = val.decode(encoding=env.get('XONSH_ENCODING'),
+                         errors=env.get('XONSH_ENCODING_ERRORS'))
+    return string + _post, val + _post, quote, end
+
+
 XONSH_TOKENS = {
     'and ', 'as ', 'assert ', 'break', 'class ', 'continue', 'def ', 'del ',
     'elif ', 'else', 'except ', 'finally:', 'for ', 'from ', 'global ',
@@ -27,6 +63,10 @@ XONSH_TOKENS = {
     '...'
 }
 
+COMPLETION_SKIP_TOKENS = {'sudo', 'time', 'man'}
+
+COMPLETION_WRAP_TOKENS = {' ',',','[',']','(',')','{','}'}
+
 BASH_COMPLETE_SCRIPT = """source {filename}
 COMP_WORDS=({line})
 COMP_LINE={comp_line}
@@ -56,13 +96,13 @@ def startswithnorm(x, start, startlow=None):
 
 
 def _normpath(p):
-    """ Wraps os.normpath() to avoid removing './' at the beginning 
+    """ Wraps os.normpath() to avoid removing './' at the beginning
         and '/' at the end. On windows it does the same with backslases
-    """   
+    """
     initial_dotslash = p.startswith(os.curdir + os.sep)
     initial_dotslash |= (ON_WINDOWS and p.startswith(os.curdir + os.altsep))
     p = p.rstrip()
-    trailing_slash = p.endswith(os.sep) 
+    trailing_slash = p.endswith(os.sep)
     trailing_slash |= (ON_WINDOWS and p.endswith(os.altsep))
     p = os.path.normpath(p)
     if initial_dotslash and p != '.':
@@ -75,6 +115,17 @@ def _normpath(p):
 
     return p
 
+def completionwrap(s):
+    """ Returns the repr of input string s if that string contains 
+    a 'problem' token that will confuse the xonsh parser
+    """
+    space = ' '
+    slash = get_sep()
+    return (_normpath(repr(s + (slash if os.path.isdir(s) else '')))
+           if COMPLETION_WRAP_TOKENS.intersection(s) else
+           s + space
+           if s[-1:].isalnum() else
+           s)
 
 class Completer(object):
     """This provides a list of optional completions for the xonsh shell."""
@@ -115,13 +166,33 @@ class Completer(object):
         -------
         rtn : list of str
             Possible completions of prefix, sorted alphabetically.
+        lprefix : int
+            Length of the prefix to be replaced in the completion
+            (only used with prompt_toolkit)
         """
         space = ' '  # intern some strings for faster appending
         slash = '/'
         dot = '.'
         ctx = ctx or {}
         prefixlow = prefix.lower()
+        _line = line
+        line = builtins.aliases.expand_alias(line)
+        # string stuff for automatic quoting
+        path_str_start = ''
+        path_str_end = ''
+        p = _path_from_partial_string(_line, endidx)
+        lprefix = len(prefix)
+        if p is not None:
+            lprefix = len(p[0])
+            prefix = p[1]
+            path_str_start = p[2]
+            path_str_end = p[3]
         cmd = line.split(' ', 1)[0]
+        while cmd in COMPLETION_SKIP_TOKENS:
+            begidx -= len(cmd)+1
+            endidx -= len(cmd)+1
+            cmd = line.split(' ', 2)[1]
+            line = line.split(' ', 1)[1]
         csc = builtins.__xonsh_env__.get('CASE_SENSITIVE_COMPLETIONS')
         startswither = startswithnorm if csc else startswithlow
         if begidx == 0:
@@ -135,20 +206,19 @@ class Completer(object):
                     s = s.rstrip() + slash
                 rtn.add(s)
             if len(rtn) == 0:
-                rtn = self.path_complete(prefix)
-            return sorted(rtn)
+                rtn = self.path_complete(prefix, path_str_start, path_str_end)
         elif prefix.startswith('${') or prefix.startswith('@('):
             # python mode explicitly
             rtn = set()
         elif prefix.startswith('-'):
-            return sorted(self._man_completer.option_complete(prefix, cmd))
+            return sorted(self._man_completer.option_complete(prefix, cmd)), lprefix
         elif cmd not in ctx:
             if cmd == 'import' and begidx == len('import '):
                 # completing module to import
-                return sorted(self.module_complete(prefix))
+                return sorted(self.module_complete(prefix)), lprefix
             if cmd in self._all_commands():
                 # subproc mode; do path completions
-                return sorted(self.path_complete(prefix, cdpath=True))
+                return sorted(self.path_complete(prefix, path_str_start, path_str_end, cdpath=True)), lprefix
             else:
                 # if we're here, could be anything
                 rtn = set()
@@ -164,8 +234,9 @@ class Completer(object):
         rtn |= {s for s in dir(builtins) if startswither(s, prefix, prefixlow)}
         rtn |= {s + space for s in builtins.aliases
                 if startswither(s, prefix, prefixlow)}
-        rtn |= self.path_complete(prefix)
-        return sorted(rtn)
+        rtn |= self.path_complete(prefix, path_str_start, path_str_end)
+        return sorted(rtn), lprefix
+
 
     def find_and_complete(self, line, idx, ctx=None):
         """Finds the completions given only the full code line and a current cursor
@@ -244,20 +315,40 @@ class Completer(object):
         startswither = startswithnorm if csc else startswithlow
         return {s for s in modules if startswither(s, prefix, prefixlow)}
 
-    def path_complete(self, prefix, cdpath=False):
+    def _quote_paths(self, paths, start, end):
+        out = set()
+        space = ' '
+        backslash = '\\'
+        double_backslash = '\\\\'
+        slash = get_sep()
+        for s in paths:
+            if (start == '' and
+                    (space in s or (backslash in s and slash != backslash))):
+                start = "'"
+                end = "'"
+            if os.path.isdir(expand_path(s)):
+                _tail = slash
+            elif end == '':
+                _tail = space
+            else:
+                _tail = ''
+            s = s + _tail
+            if end != '':
+                if "r" not in start.lower():
+                    s = s.replace(backslash, double_backslash)
+                elif s.endswith(backslash):
+                    s += backslash
+            out.add(start + s + end)
+        return out
+
+
+    def path_complete(self, prefix, start, end, cdpath=False):
         """Completes based on a path name."""
         space = ' '  # intern some strings for faster appending
-        slash = '/'
         tilde = '~'
         paths = set()
         csc = builtins.__xonsh_env__.get('CASE_SENSITIVE_COMPLETIONS')
-        if prefix.startswith("'") or prefix.startswith('"'):
-            prefix = prefix[1:]
         for s in iglobpath(prefix + '*', ignore_case=(not csc)):
-            if space in s:
-                s = repr(s + (slash if os.path.isdir(s) else ''))
-            else:
-                s = s + (slash if os.path.isdir(s) else space)
             paths.add(s)
         if tilde in prefix:
             home = os.path.expanduser(tilde)
@@ -266,7 +357,7 @@ class Completer(object):
         self._add_dots(paths, prefix)
         if cdpath:
             self._add_cdpaths(paths, prefix)
-        return {_normpath(s) for s in paths}
+        return self._quote_paths({_normpath(s) for s in paths}, start, end)
 
     def bash_complete(self, prefix, line, begidx, endidx):
         """Attempts BASH completion."""
@@ -306,8 +397,7 @@ class Completer(object):
         except subprocess.CalledProcessError:
             out = ''
 
-        space = ' '
-        rtn = {s + space if s[-1:].isalnum() else s for s in out.splitlines()}
+        rtn = set(map(completionwrap, out.splitlines()))
         return rtn
 
     def _source_completions(self):
@@ -315,7 +405,7 @@ class Completer(object):
         for f in builtins.__xonsh_env__.get('BASH_COMPLETIONS'):
             if os.path.isfile(f):
                 # We need to "Unixify" Windows paths for Bash to understand
-                if ON_WINDOWS:  
+                if ON_WINDOWS:
                     f = RE_WIN_DRIVE.sub(lambda m: '/{0}/'.format(m.group(1).lower()), f).replace('\\', '/')
                 srcs.append('source ' + f)
         return srcs
@@ -369,14 +459,25 @@ class Completer(object):
         expr = subexpr_from_unbalanced(expr, '(', ')')
         expr = subexpr_from_unbalanced(expr, '[', ']')
         expr = subexpr_from_unbalanced(expr, '{', '}')
+        _ctx = None
         try:
-            val = builtins.evalx(expr, glbs=ctx)
+            val = eval(expr, ctx)
+            _ctx = ctx
         except:  # pylint:disable=bare-except
             try:
-                val = builtins.evalx(expr, glbs=builtins.__dict__)
+                val = eval(expr, builtins.__dict__)
+                _ctx = builtins.__dict__
             except:  # pylint:disable=bare-except
                 return attrs  # anything could have gone wrong!
-        opts = dir(val)
+        _opts = dir(val)
+        # check whether these options actually work (e.g., disallow 7.imag)
+        opts = []
+        for i in _opts:
+            try:
+                v = eval('{0}.{1}'.format(expr, i), _ctx)
+                opts.append(i)
+            except:  # pylint:disable=bare-except
+                continue
         if len(attr) == 0:
             opts = [o for o in opts if not o.startswith('_')]
         else:
@@ -402,8 +503,8 @@ class Completer(object):
         self._path_checksum = path_hash
         # did aliases change?
         al_hash = hash(tuple(sorted(builtins.aliases.keys())))
-        self._alias_checksum = al_hash
         cache_valid = cache_valid and al_hash == self._alias_checksum
+        self._alias_checksum = al_hash
         pm = self._path_mtime
         # did the contents of any directory in PATH change?
         for d in filter(os.path.isdir, path):
