@@ -1,4 +1,6 @@
+# -*- coding: utf-8 -*-
 """The xonsh shell"""
+import random
 import builtins
 from warnings import warn
 
@@ -30,13 +32,15 @@ class Shell(object):
         if shell_type is not None:
             env['SHELL_TYPE'] = shell_type
         shell_type = env.get('SHELL_TYPE')
+        if shell_type == 'random':
+            shell_type = random.choice(('readline', 'prompt_toolkit'))
         if shell_type == 'prompt_toolkit':
             if not is_prompt_toolkit_available():
                 warn('prompt_toolkit is not available, using readline instead.')
                 shell_type = env['SHELL_TYPE'] = 'readline'
         # actually make the shell
         if shell_type == 'prompt_toolkit':
-            from xonsh.prompt_toolkit_shell import PromptToolkitShell
+            from xonsh.ptk.shell import PromptToolkitShell
             self.shell = PromptToolkitShell(execer=self.execer,
                                             ctx=self.ctx, **kwargs)
         elif shell_type == 'readline':
@@ -60,6 +64,6 @@ class Shell(object):
             self.ctx = ctx
         else:
             rc = env.get('XONSHRC')
-            self.ctx = xonshrc_context(rcfile=rc, execer=self.execer)
+            self.ctx = xonshrc_context(rcfiles=rc, execer=self.execer)
         builtins.__xonsh_ctx__ = self.ctx
         self.ctx['__name__'] = '__main__'
